module EZDyn
  class Client
    # @private
    def add_pending_change(chg)
      @pending_changes ||= []
      @pending_changes << chg
    end

    # @private
    def pending_change_zones
      @pending_changes.collect(&:zone).uniq
    end

    # List currently pending changes (optionally per zone).
    #
    # @param zone [String] (Optional) If specified, only return pending changes
    #   for the named zone.
    # @return [Array] Array of [Change] objects awaiting commit.
    def pending_changes(zone: nil)
      if zone.nil?
        @pending_changes
      else
        zone = Zone.new(client: self, name: zone)
        @pending_changes.select do |pc|
          pc.zone.name == zone.name
        end
      end
    end

    # @private
    def clear_pending_changes(zone: nil)
      if zone.nil?
        @pending_changes = []
      else
        @pending_changes.delete_if do |pc|
          pc.zone.name == zone.to_s
        end
      end
    end
  end

  # Generic base class for any pending change.
  class Change
    # Returns the zone this change is part of.
    #
    # @return [Zone] The zone this change is part of.
    def zone
      @record.zone
    end
  end

  # A pending record creation.
  class CreateChange < Change
    # @private
    def initialize(record:)
      @record = record
    end

    # Returns a string representation of the change.
    #
    # @return [String] A string representation of this change.
    def to_s
      "CREATE #{@record.fqdn}.   #{@record.ttl} #{@record.type}   #{@record.value}"
    end
  end

  # A pending record update.
  class UpdateChange < Change
    # @private
    def initialize(records:, new_records:)
      @records = records.map(&:sync!)
      @new_records = new_records
    end

    def zone
      @new_records.first.zone
    end

    # Returns a string representation of the change.
    #
    # @return [String] A string representation of this change.
    def to_s
      ttl_string =
        if @records.map(&:ttl).min == @new_records.map(&:ttl).min
          @records.map(&:ttl).min
        else
          "(( #{@records.map(&:ttl).min} -> #{@new_records.map(&:ttl).min} ))"
        end

      value_string =
        if @records.map(&:value).join(',') == @new_records.map(&:value).join(',')
          @records.map(&:value).join(',')
        else
          "(( #{@records.map(&:value).join(',')} -> #{@new_records.map(&:value).join(',')} ))"
        end

      "UPDATE #{@new_records.first.fqdn}.   #{ttl_string} #{@new_records.first.type}   #{value_string}"
    end
  end

  # A pending record deletion.
  class DeleteChange < Change
    # @private
    def initialize(record:)
      @record = record.sync!
    end

    # Returns a string representation of the change.
    #
    # @return [String] A string representation of this change.
    def to_s
      "DELETE #{@record.fqdn}.   #{@record.ttl} #{@record.type}   #{@record.value}"
    end
  end
end
