require 'net/http'

module EZDyn
  # The main class for Dyn REST API interaction.
  #
  # For more information about the API, see
  # {https://help.dyn.com/dns-api-knowledge-base/ the official documentation}.
  #
  class Client
    # Initializes a new Dyn REST API client.
    #
    # @param customer_name [String] API customer name.
    # @param username [String] API username.
    # @param password [String] API password.
    # @return [Client] Dyn REST API client.
    def initialize(customer_name: ENV['DYN_CUSTOMER_NAME'], username: ENV['DYN_USERNAME'], password: ENV['DYN_PASSWORD'])
      @customer_name = customer_name
      @username = username
      @password = password

      if @customer_name.nil? or @username.nil? or @password.nil?
        EZDyn.info { "Some credentials are missing" }
        raise "Missing credentials"
      end

      @base_url = URI('https://api2.dynect.net/')
      @headers = {
        'Content-Type' => 'application/json',
      }

      @logged_in = false
    end

    # @private
    def build_url(path)
      path = path.to_s
      path = "/#{path}" unless path.start_with?('/')
      path = "/REST#{path}" unless path.start_with?('/REST')
      @base_url.merge(path)
    end

    # Performs an actual REST call.
    #
    # @param method [String] HTTP method to use, eg "GET", "POST", "DELETE", "PUT".
    # @param uri [String] Relative API URI to call.
    # @param payload [Hash] JSON data payload to send with request.
    # @return [Response] Response object.
    def call_api(method: "get", uri:, payload: {}, max_attempts: EZDyn::API_RETRY_MAX_ATTEMPTS)
      EZDyn.debug { "API CALL: #{method.to_s.upcase} #{uri} #{ ( payload || {} ).to_json }" }
      self.login if not self.logged_in? and uri != "Session"

      payload_str = payload.to_json
      if payload == {}
        payload_str = nil
      end

      response = nil
      begin
        EZDyn.debug { "About to make REST request to #{method.to_s.upcase} #{uri}" }

        request_url = build_url(uri)

        request = Net::HTTP.const_get(method.capitalize).new(request_url)
        @headers.each do |name, value|
          request[name] = value
        end
        request.body = payload_str if payload_str

        http = Net::HTTP.new(request_url.host, request_url.port)
        http.use_ssl = true if request_url.scheme == 'https'
        response = Response.new(http.start { |http| http.request(request) })

        if response.delayed?
          wait = EZDyn::API_RETRY_DELAY_SECONDS
          max_attempts.times do |retry_count|
            EZDyn.debug { "Async API call response retrieval, attempt #{retry_count + 1}" }
            EZDyn.debug { "Waiting for #{wait} seconds" }
            sleep wait

            response = self.call_api(uri: "/REST/Job/#{response.job_id}")
            break if response.success?

            EZDyn.debug { "Async response status: #{response.status}" }
            wait += (retry_count * EZDyn::API_RETRY_BACKOFF)
          end
        end

        EZDyn.debug { "Call was successful" }
      rescue => e
        EZDyn.info { "REST request to #{uri} threw an exception: #{e}" }
        raise "Got an exception: #{e}"
      end

      response
    end

    # Signals whether the client has successfully logged in.
    #
    # @return [Boolean] True if the client is logged in.
    def logged_in?
      @logged_in
    end

    # Begin a Dyn REST API session. This method will be called implicitly when required.
    def login
      EZDyn.debug { "Logging in..." }
      response = call_api(
        method: "post",
        uri: "Session",
        payload: {
          customer_name: @customer_name,
          user_name: @username,
          password: @password,
        }
      )

      EZDyn.debug { "Response status: #{response.status}" }

      if response.success?
        @headers['Auth-Token'] = response.data["token"]
        @logged_in = true
      else
        raise "Login failed"
      end
    end

    # End the current Dyn REST API session.
    def logout
      call_api(
        method: "delete",
        uri: "Session"
      )

      @headers.delete('Auth-Token')
      @logged_in = false
    end

    # @private
    def build_uri(type:, fqdn:, id: nil)
      EZDyn.debug { "Client.build_uri( type: #{type}, fqdn: #{fqdn}, id: #{id} )" }
      "#{RecordType.find(type).uri_name}/#{self.guess_zone(fqdn: fqdn)}/#{fqdn}/#{id}"
    end

    # @private
    def fetch_uri_data(uri:)
      EZDyn.debug { "Client.fetch_uri_data( uri: #{uri} )" }
      response = call_api(uri: uri)
      if response.success?
        EZDyn.debug { "fetch_uri_data success!" }
        return response.data
      else
        EZDyn.debug { "fetch_uri_data failure!" }
        return []
      end
    end

    # Fetches a record if you know the type, FQDN, and ID.
    #
    # @param type [EZDyn::RecordType, String, Symbol] The record type to be fetched.
    # @param fqdn [String] FQDN of the record to fetch.
    # @param id [String] Dyn API record ID of the record to fetch.
    # @return [EZDyn::Record] Newly created, synced Record object.
    def fetch_record(type:, fqdn:, id:)
      EZDyn.debug { "Client.fetch_record( type: #{type}, fqdn: #{fqdn}, id: #{id} )" }
      data = self.fetch_uri_data(uri: self.build_uri(type: type, fqdn: fqdn, id: id))
      if data == []
        nil
      else
        Record.new(client: self, raw: data)
      end
    end

    # Fetches all records for a particular FQDN (and record type).
    #
    # @param type [EZDyn::RecordType, String, Symbol] Desired record type. Use
    #   `:any` to fetch all records.
    # @param fqdn [String] FQDN of the records to fetch.
    # @return [Array<EZDyn::Record>] An array of synced Record objects.
    def records_for(type: :any, fqdn:)
      EZDyn.debug { "Client.records_for( type: #{type}, fqdn: #{fqdn} )" }
      self.fetch_uri_data(uri: self.build_uri(type: type, fqdn: fqdn)).
        collect { |uri| Record.new(client: self, uri: uri) }
    end

    # Fetches all records for a zone. NOTE: This can take very long, and probably isn't what you want.
    #
    # @param zone [String] The Zone to lookup.
    def records_in_zone(zone:)
      EZDyn.debug { "Client.records_in_zone( zone: #{zone}" }
      self.fetch_uri_data(uri: "/AllRecord/#{zone}").
        collect{ |uri| Record.new(client: self, uri: uri) }
    end

    # Signals if a particular record exists.
    #
    # @param type [EZDyn::RecordType, String, Symbol] RecordType of record to
    #   check for. `:any` will match any record type.
    # @param fqdn [String] FQDN of records to check for. Required.
    # @param id [String] API record ID of the record to check for. Optional.
    # @return [Boolean] Returns true if any such record exists.
    def exists?(type: :any, fqdn:, id: nil)
      EZDyn.debug { "Client.exists?( type: #{type}, fqdn: #{fqdn}, id: #{id} )" }
      if not id.nil? and type != :any
        EZDyn.debug { "Fetching a single record" }
        self.fetch_record(type: type, fqdn: fqdn, id: id) != []
      else
        EZDyn.debug { "Fetching records_for #{type} #{fqdn}" }
        self.records_for(type: type, fqdn: fqdn).count > 0
      end
    end
  end
end
