module EZDyn
  class Client
    # Calls the Dyn API to create a new record.
    #
    # @note As a side effect upon success, this method creates a [CreateChange]
    #   object in the `pending_changes` array.
    #
    # @raise [RuntimeError] if the record could not be created.
    # @param type [RecordType, String, Symbol] Type of record to create.
    # @param fqdn [String] FQDN of the record to create.
    # @param value [String, Array] Value(s) to submit for the record data.
    # @param ttl [String, Integer] TTL value (optional).
    # @return [Record] A Record object filled with the values returned by the API.
    def create(type:, fqdn:, value:, ttl: nil)
      EZDyn.debug { "Client.create( type: #{type}, fqdn: #{fqdn}, value: #{value}, ttl: #{ttl} )" }

      ttl = ( ttl || Record::DefaultTTL ).to_i
      values = Array(value)

      return values.map do |val|
        value_key = Array(RecordType.find(type).value_key)
        split_val = val.split(' ', value_key.length)
        response = self.call_api(
          method: "post",
          uri: self.build_uri(type: type, fqdn: fqdn),
          payload: { rdata: value_key.zip(split_val).to_h, ttl: ttl }
        )

        if not response.success?
          raise "Failed to create record: #{response.simple_message}"
        end

        record = Record.new(client: self, raw: response.data)
        self.add_pending_change(CreateChange.new(record: record))
        record
      end
    end

    # Calls the Dyn API to update or create a record. Could also be called
    # `upsert`.
    #
    # @note As a side effect upon success, this method creates an [UpdateChange]
    #   or a [CreateChange] object in the `pending_changes` array.
    #
    # @raise [RuntimeError] if the record could not be created or updated.
    # @param record [Record] A Record object for the record to be updated.
    #   Either this parameter or the `type` and `fqdn` parameters are
    #   required.
    # @param type [RecordType, String, Symbol] Type of record to update/create (not required if `record` is provided).
    # @param fqdn [String] FQDN of the record to update/create (not requried if `record` is provided).
    # @param value [String, Array] New value(s) to submit for the record data (optional if updating TTL and record already exists).
    # @param ttl [String, Integer] New TTL value (optional).
    # @return [Record] A Record object filled with the values returned by the API.
    def update(record: nil, type: nil, fqdn: nil, value: nil, ttl: nil)
      EZDyn.debug { "Client.update( record: #{record.nil? ? nil : "Record{#{record.record_id}}"}, type: #{type}, fqdn: #{fqdn}, value: #{value}, ttl: #{ttl} )" }

      values = Array(value)

      if record.nil?
        if type.nil? or fqdn.nil?
          raise "Cannot update a record without a Record object or both record type and FQDN"
        end
        records = self.records_for(type: type, fqdn: fqdn)
      else
        records = [record]
      end

      response = nil
      if records.count.zero? && (fqdn.nil? || type.nil? || value.nil?)
        raise "Record doesn't exist, and insufficient information to create it was given"
      end

      response = self.call_api(
        method: "put",
        uri: self.build_uri(type: type, fqdn: fqdn),
        payload: {
          "#{type}Records" => values.map do |val|
            value_key = Array(RecordType.find(type).value_key)
            split_val = val.split(' ', value_key.length)
            { rdata: value_key.zip(split_val).to_h, ttl: ttl || records.map(&:ttl).min }
          end
        }
      )

      if not response.success?
        raise "Could not update: #{response.simple_message}"
      end

      new_records = response.data.map { |res| Record.new(client: self, raw: res) }
      self.add_pending_change(UpdateChange.new(records: records, new_records: new_records))

      return new_records
    end

    # Delete a record.
    #
    # @note As a side effect upon success, this method creates a [DeleteChange]
    #   object in the `pending_changes` array.
    #
    # @raise [RuntimeError] if record does not exist.
    # @raise [RuntimeError] if record cannot be deleted.
    # @param record [Record] The Record object of the record to be deleted.
    def delete(record:)
      EZDyn.debug { "Client{}.delete( record: Record{#{record.record_id}} )" }
      if not record.sync!.exists?
        raise "Nothing to delete"
      end

      response = self.call_api(
        method: "delete",
        uri: record.uri
      )

      if not response.success?
        raise "Could not delete: #{response.simple_message}"
      end

      self.add_pending_change(DeleteChange.new(record: record))
    end

    # Deletes all records of a specified type and FQDN. Specify type `:any`
    # (or don't specify `type` at all) to delete all records for a FQDN.
    #
    # @note As a side effect upon success, this method creates [DeleteChange]
    #   objects in the `pending_changes` array for each record deleted.
    #
    # @param type [RecordType,String,Symbol] The type of record(s) to delete.
    #   Defaults to `:any`.
    # @param fqdn [String] The FQDN of the record(s) to delete.
    def delete_all(type: :any, fqdn:)
      EZDyn.debug { "Client{}.delete_all( type: #{type}, fqdn: #{fqdn} )" }
      self.records_for(type: type, fqdn: fqdn).each do |record|
        record.delete!
      end
    end

    # Rolls back any pending changes to a zone or to all zones.
    #
    # @param zone [String] The zone name to roll back. By default all pending
    #   changes from any zone will be rolled back.
    def rollback(zone: nil)
      EZDyn.debug { "Client{}.rollback( zone: #{zone} )" }

      zones = zone.nil? ? self.pending_change_zones : [Zone.new(client: self, name: zone)]

      zones.each do |zone|
        EZDyn.debug { " - rolling back Zone{#{zone.name}}" }
        response = self.call_api(method: "delete", uri: "ZoneChanges/#{zone}")
        if response.success?
          self.clear_pending_changes(zone: zone)
        else
          EZDyn.debug { " - failed to roll back Zone{#{zone.name}}: #{response.simple_message}" }
          raise "Failed to roll back zone #{zone.name}"
        end
      end
    end

    # Commits any pending changes to a zone or to all zones with an optional update message.
    #
    # @raise [RuntimeError] if any commit was unsuccessful.
    # @param zone [String] The zone name to commit. By default all pending
    #   changes from any zone will be committed.
    # @param message [String] If supplied, this message will be used for the
    #   zone update notes field.
    def commit(zone: nil, message: nil)
      EZDyn.debug { "Client{}.commit( zone: #{zone} )" }
      payload = { publish: true }
      payload[:notes] = message if not message.nil?

      zones = zone.nil? ? self.pending_change_zones : [Zone.find(zone)]

      zones.each do |zone|
        EZDyn.debug { " - committing Zone{#{zone.name}}" }
        response = self.call_api(
          method: "put",
          uri: "Zone/#{zone}",
          payload: payload
        )

        if response.success?
          self.clear_pending_changes(zone: zone)
        else
          EZDyn.debug { " - failed to commit Zone{#{zone.name}}: #{response.simple_message}" }
          raise "Could not commit zone #{zone.name}: #{response.simple_message}"
        end
      end
    end
  end
end
