module EZDyn
  # Abstraction of Dyn REST API DNS records.
  class Record
    # Default TTL (time to live) for new records
    DefaultTTL = 300

    # @private
    def initialize(client:, raw: nil, uri: nil, type: nil, fqdn: nil, value: nil, ttl: nil, record_id: nil)
      @client = client
      @exists = nil
      @type = RecordType.find(type)
      @fqdn = fqdn
      @value = value
      @ttl = ttl
      @in_sync = false

      if not raw.nil?
        self.sync_raw(raw)

      elsif not uri.nil?
        @uri = uri.gsub(%r{^/?(REST/)?}, "")
        if @uri !~ %r{^[A-Za-z]+Record/[^/]+/[^/]+/[0-9]+}
          raise "Invalid Record URI: '#{uri}'"
        end

        type_uri_name, zone, fqdn, record_id = @uri.split('/')

        @type = RecordType.find(type_uri_name)
        @fqdn = fqdn
        @record_id = record_id
        @zone = Zone.new(client: @client, name: zone)
        @exists = true
      end
    end

    # Returns the record type.
    def type
      self.sync! if @type.nil?
      @type
    end

    # Returns the TTL of this record.
    def ttl
      self.sync! if @ttl.nil?
      @ttl
    end

    # Returns the FQDN of this record.
    def fqdn
      self.sync! if @fqdn.nil?
      @fqdn
    end

    # Returns the record data value.
    def value
      self.sync! if @value.nil?
      @value
    end

    # Returns the Dyn REST API ID for this record.
    def record_id
      self.sync! if @record_id.nil?
      @record_id
    end

    # Returns the zone of this record.
    def zone
      @zone ||= @client.guess_zone(fqdn: self.fqdn)
    end

    # Returns whether this record existed at its last sync.
    def exists?
      @exists
    end

    # Returns whether this record has been synced.
    def in_sync?
      @in_sync
    end

    # @private
    def uri
      if @uri.nil?
        "#{self.type.uri_name}/#{self.zone.name}/#{self.fqdn}/#{self.record_id}"
      else
        @uri
      end
    end

    # Attempts to sync the record to the API.
    #
    # @raise [RuntimeError] if the record could not be synced.
    # @raise [RuntimeError] if more than one record matches this record.
    # @raise [RuntimeError] if returned data was not in an expected format.
    def sync!
      return self if self.in_sync?

      data = @client.fetch_uri_data(uri: self.uri)
      if data.is_a? Array
        if data.count == 0
          @in_sync = true
          @exists = false
          return self

        elsif data.count > 1
          raise "More than one record was found"

        end
      end

      if data.is_a? Array and data.count == 1
        data = data.first
      end

      if data.is_a? Hash
        self.sync_raw(data)

      else
        raise "Unrecognized data: #{data.class} #{data}"
      end

      self
    end

    # @private
    def sync_raw(raw)
      @zone = Zone.new(client: @client, name: raw["zone"])
      @ttl = raw["ttl"]
      @fqdn = raw["fqdn"]
      @type = RecordType.find(raw["record_type"])
      @record_id = raw["record_id"].to_s
      @value = Array(@type.value_key).map { |k| raw["rdata"][k] }.join(' ')
      @in_sync = true
      @exists = true
    end

    # Attempt to delete this record.
    def delete!
      @client.delete(record: self)
    end
  end
end
