module EZDyn
  # Encapsulates the properties of supported record types
  class RecordType
    # @private
    @@types = []

    # Check if a type is valid.
    #
    # @return [Boolean] Whether the type given is valid.
    def self.valid_type?(t)
      not RecordType.find(t).nil?
    end

    # The URI path element that signals this record type.
    attr_reader :uri_name

    # The dict key for this record type's `rdata` value.
    attr_reader :value_key

    # RecordType constructor. In general there is no reason to call this method
    # directly as all known record types are created at file load time. Instead
    # the #find method is the preferred way to fetch a RecordType object.
    #
    # @param name [Symbol] The canonical type of the record.
    # @param uri_name [String] URI path element for the record type.
    # @param value_key [String] Record data value dict key.
    def initialize(name:, uri_name:, value_key:)
      EZDyn.debug { "RecordType.new( name: #{name}, uri_name: #{uri_name}, value_key: #{value_key} )" }
      @name =  name
      @uri_name = uri_name
      @value_key = value_key
    end

    # Converts several possible representations of a RecordType into an
    # appropriate class instance. Pass in a RecordType instance, a String or
    # Symbol of the type name or the String fragment of a REST API URI.
    #
    # @param to_find [EZDyn::RecordType, String, Symbol] Representation of the
    #   paramter to be found.
    # @return [EZDyn::RecordType] The class instance requested or `nil` if none
    #   is found.
    def self.find(to_find)
      EZDyn.debug { "RecordType#find( [#{to_find.class}] #{to_find} )" }
      if to_find.is_a? RecordType
        return to_find

      else
        return @@types.find { |ctype| ctype.name == to_find.to_s.upcase } ||
                 @@types.find { |ctype| ctype.uri_name.downcase == to_find.to_s.downcase }
      end

      EZDyn.debug { "No such RecordType was found" }
    end

    # Provides an all-caps String representation of the record type, eg 'A','CNAME'.
    #
    # @return [String] The name of the record type.
    def name
      @name.to_s.upcase
    end

    # Converts the RecordType to the all-caps String of the record type.
    #
    # @return [String] The displayable String representation.
    def to_s
      self.name
    end

    # initialize cache of supported types
    [
      { name: :any,   uri_name: "ANYRecord",   value_key: nil       },
      { name: :a,     uri_name: "ARecord",     value_key: "address" },
      { name: :cname, uri_name: "CNAMERecord", value_key: "cname"   },
      { name: :aaaa,  uri_name: "AAAARecord",  value_key: "address" },
      { name: :soa,   uri_name: "SOARecord",   value_key: nil },
      { name: :mx,    uri_name: "MXRecord",    value_key: ["preference", "exchange"] },
      { name: :ns,    uri_name: "NSRecord",    value_key: "nsdname" },
      { name: :txt,   uri_name: "TXTRecord",   value_key: "txtdata" },
      { name: :spf,   uri_name: "SPFRecord",   value_key: nil }
    ].each do |args|
      @@types << EZDyn::RecordType.new(**args)
    end
  end
end
