module EZDyn
  # Abstraction of a Dyn REST API response.
  class Response
    # @private
    def initialize(response)
      @response = response
      EZDyn.debug { "API RESPONSE: #{@response.body}" }
      if @response.body =~ %r{^/REST/Job/[0-9]+$}
        EZDyn.debug { "Response delayed! Try again later!" }
        @raw = {
          "status" => "delayed",
          "job_id" => @response.body.split('/').last,
          "msgs" => [],
          "data" => {},
        }
      else
        @raw = JSON.parse(@response.body)
      end
    end

    # Returns the status message of the response.
    #
    # @return [String] The status field of the response.
    def status
      @raw["status"]
    end

    # Was the response successful?
    #
    # @return [Boolean] Returns true if the response was successful.
    def success?
      self.status == "success"
    end

    # Do we have to wait for the response?
    #
    # @return [Boolean] Returns true if the response is delayed and you
    #   must wait and try again.
    def delayed?
      self.status == "delayed"
    end

    # The job ID returned by the response.
    #
    # @return [String] The job ID of the response.
    def job_id
      @raw["job_id"]
    end

    # The payload data from the response.
    #
    # @return [Hash] The payload data of the response.
    def data
      @raw["data"]
    end

    # Full version of the descriptive response message with log levels and
    # error codes.
    #
    # @return [String] Full descriptive message of the response with error
    #   codes and log levels.
    def full_message
      @raw["msgs"].collect do |msg|
        "[#{msg["SOURCE"]}] #{msg["LVL"]} (#{msg["ERR_CD"]}): #{msg[msg["LVL"]]}"
      end.join("\n")
    end

    # The simple descriptive message of the response from the API.
    #
    # @return [String] Descriptive message of the response.
    def simple_message
      @raw["msgs"].collect { |m| m[m["LVL"]] }.join("\n")
    end

    # All error codes returned by the response.
    #
    # @return [Array<String>] An array of error codes returned by the response.
    def error_codes
      @raw["msgs"].collect { |m| m["ERR_CD"] }.compact.uniq
    end
  end
end
