module EZDyn
  # Abstraction of Dyn REST API DNS Zones
  class Zone
    attr_reader :name, :type, :serial, :serial_style

    # @private
    def initialize(client:, name: nil, type: nil, serial: nil, serial_style: nil, raw: nil, uri: nil)
      EZDyn.debug { "Zone.new( client: Client{}, name: #{name}, type: #{type}, serial: #{serial}, serial_style: #{serial_style}, raw: #{raw.nil? ? nil : raw.to_json}, uri: #{uri} )" }
      @client = client
      @name = name
      @type = type
      @serial = serial
      @serial_style = serial_style
      @in_sync = false

      if not raw.nil?
        self.sync_raw(raw)
      end

      if not uri.nil?
        uri.gsub!(%r{^/?(REST/)?}, '')
        if uri =~ %r{^Zone/([^/]+)/?$}
          @name = $1
        end
      end
    end

    # Returns the zone type.
    def type
      self.sync! if @type.nil?
      @type
    end

    # Returns the serial number of the zone.
    def serial
      self.sync! if @serial.nil?
      @serial
    end

    # Returns the style of serial number in use by this zone.
    def serial_style
      self.sync! if @serial_style.nil?
      @serial_style
    end

    # @private
    def uri
      "/Zone/#{self.name}"
    end

    # Indicates whether the object has been synced with the API.
    def in_sync?
      @in_sync
    end

    # Attempt to sync the object with the API.
    #
    # @raise [RuntimeError] if the object does not exist or could not be synced.
    def sync!
      EZDyn.debug { "Zone{#{self.name}}.sync!" }
      return self if self.in_sync?

      data = @client.fetch_uri_data(uri: self.uri)
      if data.is_a? Hash
        sync_raw(data)
      else
        raise "Failed to sync zone #{self.name}"
      end

      self
    end

    # @private
    def sync_raw(raw)
      EZDyn.debug { "Zone{#{self.name}}.sync_raw( #{raw.nil? ? nil : raw.to_json} )" }
      @name = raw["zone"]
      @type = raw["zone_type"]
      @serial = raw["serial"]
      @serial_style = raw["serial_style"]
      @in_sync = true
    end

    # Returns the name of the zone for display purposes.
    def to_s
      self.name
    end
  end

  class Client
    # @private
    def fetch_zones
      EZDyn.debug { "Client.fetch_zones()" }
      @zones = self.fetch_uri_data(uri: '/Zone/').
        collect { |uri| Zone.new(client: self, uri: uri) }
    end

    # List all DNS zones known to this client.
    #
    # @return [Array<Zone>] An array of Zone objects.
    def zones
      EZDyn.debug { "Client.zones()" }
      @zones ||= self.fetch_zones
    end

    # Match the given FQDN to a zone known to this client.
    #
    # @return [Zone] The appropriate Zone object, or nil if nothing matched.
    def guess_zone(fqdn:)
      EZDyn.debug { "Client.guess_zone( fqdn: #{fqdn} )" }
      self.zones.find { |z| fqdn.downcase =~ /#{z.name.downcase}$/ }
    end
  end
end
